/**
 * omp_config.c
 */

#include "omp.h"
#include <ti/csl/csl_chip.h>
#include <ti/csl/csl_cache.h>
#include <ti/csl/csl_cacheAux.h>
#include <ti/csl/csl_msmc.h>
#include <ti/csl/csl_msmcAux.h>

extern cregister volatile unsigned int DNUM;

#define MSMCSRAM_START_ADDR     (0x0C000000)
#define MSMCSRAM_NC_SIZE        (0x00100000)    /* 1 MB */

#if defined (C6678)
#define MSMCSRAM_SIZE           (0x00400000)    /* 4 MB */
#define MSMCSRAM_NC_START_ADDR  (0xA0000000)
#define DDR_START_ADDR          (0x80000000)
#define DDR_SIZE                (0x20000000)    /* 512 MB */
#elif defined (C6670)
#define MSMCSRAM_SIZE           (0x00200000)    /* 2 MB */
#define MSMCSRAM_NC_START_ADDR  (0xA0000000)
#define DDR_START_ADDR          (0x80000000)
#define DDR_SIZE                (0x20000000)    /* 512 MB */
#elif defined (C6657)
#define MSMCSRAM_SIZE           (0x00100000)    /* 1 MB */
#define MSMCSRAM_NC_START_ADDR  (0xA0000000)
#define DDR_START_ADDR          (0x80000000)
#define DDR_SIZE                (0x20000000)    /* 512 MB */
#elif defined (C6636)
#define MSMCSRAM_SIZE           (0x00600000)    /* 6 MB */
#define MSMCSRAM_NC_START_ADDR  (0xE0000000)
#define DDR_START_ADDR          (0xA0000000)
#define DDR_SIZE                (0x20000000)    /* 512 MB */
#else
#error "Device not supported"
#endif

#define OMP_NUM_CORES       (8)
#define OMP_MASTER_CORE_IDX (0)

#pragma CODE_SECTION(__TI_omp_reset, ".text:omp:reset")
void __TI_omp_reset(void)
{
    /* Configure caches */
    CACHE_setL1DSize(CACHE_L1_32KCACHE);
    CACHE_setL1PSize(CACHE_L1_32KCACHE);
    CACHE_setL2Size(CACHE_0KCACHE);

    /* OMP runtime requires a portion of MSMCSRAM to be non-cached. Since it is 
     * not possible to disable caching in the MSMCSRAM address range, we need a 
     * 2 step process:
     * 1. Map a portion of MSMCSRAM into a range that can be marked as
     *    non-cached. This is done using the MPAX register
     * 2. Annotate the mapped section as non-cached using the appropriate
     *    MAR register for that memory range
     * All accesses to MSMCSRAM through the mapped address range will not
     * be cached.
     */
    __TI_setMPAX(3, MSMCSRAM_NC_START_ADDR, 
                    MSMCSRAM_START_ADDR, 0x13 /* 1 MB */);
    __TI_omp_disable_caching(MSMCSRAM_NC_START_ADDR, MSMCSRAM_NC_SIZE);

    /* Annotate MSMCSRAM and DDR as cached + prefetch + write through */
    __TI_omp_enable_caching(MSMCSRAM_START_ADDR, MSMCSRAM_SIZE);
    __TI_omp_enable_caching(DDR_START_ADDR, DDR_SIZE);
}


