/*
 *  ======== getSects ========
 */
function getSects()
{
    return "ti/runtime/openmp/linkcmd.xdt";
}

/*
 *  ======== getLibs ========
 */
function getLibs( prog )
{
    var pkg = this;
    var Settings = pkg.Settings;
 
    var suffix = prog.build.target.findSuffix(pkg);
    var debugFlag;

    switch (Settings.debugFlag) 
    {
        case Settings.DEBUG:
            debugFlag = "_debug";
            break;
        case Settings.RELEASE:
            debugFlag = "_release";
            break;
        default:
            throw new Error("debugFlag not set");
    }

    var target;
    if( String(Program.cpu.deviceName).search("6678") != -1)
    {
        target = "_c6678";
    }    
    else if( String(Program.cpu.deviceName).search("6670") != -1)
    {
        target = "_c6670";
    }
    else if( String(Program.cpu.deviceName).search("6657") != -1)
    {
        target = "_c6657";
    }
    else if( String(Program.cpu.deviceName).search("663") != -1)
    {
        target = "_c6636";
    }
    else
    {
        Program.$logError("Device:" + Program.cpu.deviceName + 
                          " not supported", this);
    }

    lib = "lib/" + pkg.$name.replace(/\./g, "_") + target + debugFlag +  
                "_" + suffix + ".a" + suffix;

    if (java.io.File(this.packageBase + lib).exists())
    {
        return lib;
    }
    else
    {
         /* could not find any library, throw exception */
         throw new Error("Library not found: " + lib);
    }
}

/* Set up flags in preparation for calls to getLibs() in OEM/OMP */
function close()
{
    var profile = Program.build.profile;

    var ompSettings = this.Settings;

    if (profile.search("debug")!=-1)
    {
        ompSettings.debugFlag=ompSettings.DEBUG;
    }
    else
    {
        ompSettings.debugFlag=ompSettings.RELEASE;
    }

    if (ompSettings.usingRtsc == true)
    {
        var OpenMP = xdc.module("ti.runtime.ompbios.OpenMP");
        OpenMP.qmssInit = ompSettings.runtimeInitializesQmss;
    }

    if( String(Program.cpu.deviceName).search("6678") != -1)
    {
        print("\tBuilding for device: " + Program.cpu.deviceName);
    }
    else if( String(Program.cpu.deviceName).search("6670") != -1)
    {
        print("\tBuilding for device: " + Program.cpu.deviceName);
    }
    else if( String(Program.cpu.deviceName).search("6657") != -1)
    {
        print("\tBuilding for device: " + Program.cpu.deviceName);
    }
    else if( String(Program.cpu.deviceName).search("663") != -1)
    {
        print("\tBuilding for device: " + Program.cpu.deviceName);
        var cslSettings = xdc.useModule ('ti.csl.Settings');
        cslSettings.deviceType = "k2h";
    }
    else
    {
        Program.$logError("Device:" + Program.cpu.deviceName + 
                          " not supported", this);
    }
}

