/*
 * Description of Memory Regions used by the OMP runtime
 *
 * L2SRAM               Local Level 2 (64K of L2 configured as SRAM)
 * MSMCSRAM             Shared Level 2 internal (MCMC configured as level 2 
 *                      memory, cached in L1)
 * MSMCSRAM_NOCACHE     Shared Level 1 internal (MSMC configured as level 1 
 *                      memory i.e. it is not cached)
 */
metaonly module Platform inherits xdc.platform.IPlatform {

    config ti.platforms.generic.Platform.Instance CPU =
        ti.platforms.generic.Platform.create("CPU", {
            clockRate:      1000,                                       
            catalogName:    "ti.catalog.c6000",
            deviceName:     "TMS320C6678",
            customMemoryMap: [
                ["L2SRAM",    
                                {name: "L2SRAM",  base: 0x00800000, 
                                len: 0x00060000, access: "RW"}],
                ["OMP_MSMC_NC_VIRT",   
                                {name: "OMP_MSMC_NC_VIRT", base: 0xA0000000, 
                                len: 0x00020000, access: "RW"}],
                ["OMP_MSMC_NC_PHY",   
                               {name: "OMP_MSMC_NC_PHY", base: 0x0C000000, 
                                len: 0x00020000, access: "RW"}],
                ["MSMCSRAM",   
                                {name: "MSMCSRAM", base: 0x0C020000, 
                                len: 0x003E0000, access: "RWX"}],
                ["DDR3",   
                                {name: "DDR3", base: 0x80000000, 
                                len: 0x20000000, access: "RWX"}],
            ],
            l2Mode:"128k",
            l1PMode:"32k",
            l1DMode:"32k",
    });
    
instance :
    
    override config string codeMemory  = "MSMCSRAM";   
    override config string dataMemory  = "DDR3";
    override config string stackMemory = "L2SRAM";
    
}
